import logging

AUTH_ENDPOINT = "/api-token-auth/"
TEAM_ENDPOINT = "/account/teams/"
VALIDATE_CONSUMER_SUBMISSION_ENDPOINT = "/orcabase/consumer/submissions/validate/"
SUBMIT_CONSUMER_SUBMISSION_ENDPOINT = "/orcabase/consumer/submissions/submit/"
VALIDATE_SERVICEOWNER_SUBMISSION_ENDPOINT = "/orcabase/serviceowner/services/validate/"
SUBMIT_SERVICEOWNER_SUBMISSION_ENDPOINT = "/orcabase/serviceowner/services/"
SUBMIT_SERVICEOWNER_SUBMISSION_DOCS_ENDPOINT = "/orcabase/serviceowner/services/$uuid/docs/"
SERVICE_ITEMS_ENDPOINT = "/orcabase/serviceowner/service_items/"
CHANGE_INSTANCES_ENDPOINT = "/orcabase/serviceowner/change_instances/"
DEPLOYED_ITEMS_ENDPOINT = "/orcabase/serviceowner/deployed_items/"
SERVICE_CONFIG_ENDPOINT = "/orcabase/serviceowner/service_configs/"

RETRY_TIMES = 1  # number of times request will be retried

LOG_FORMAT = "%(asctime)s - %(levelname)s - %(name)s - %(message)s"
LOG_LEVEL = logging.INFO

logging.basicConfig(format=LOG_FORMAT, level=LOG_LEVEL)
logger = logging.getLogger("netorca_sdk")



import json

import pytest
from unittest.mock import ANY

from netorca_sdk.exceptions import NetorcaException
from netorca_sdk.netorca import Netorca


def test_get_service_items_no_filters(netorca, auth_mock):
    auth_mock.get.return_value.status_code = 200
    auth_mock.get.return_value.json.return_value = {'results': []}

    result = netorca.get_service_items()

    auth_mock.get.assert_called_once_with(url=ANY, authentication_required=True, filters=None)


def test_get_service_items_error(auth_mock):
    auth_mock.get.return_value.status_code = 500
    auth_mock.get.return_value.json.return_value = {'error': 'Some error'}

    netorca = Netorca(auth=auth_mock)

    with pytest.raises(NetorcaException, match="Could not fetch Service Items due to: {'error': 'Some error'}"):
        netorca.get_service_items()


def test_get_service_items_with_filters(netorca, auth_mock):
    auth_mock.get.return_value.status_code = 200
    auth_mock.get.return_value.json.return_value = {'results': []}
    filters = {'service_name': 'test_service'}

    result = netorca.get_service_items(filters=filters)

    auth_mock.get.assert_called_once_with(url=ANY, authentication_required=True, filters=filters)


def test_get_deployed_items_no_filters(auth_mock):
    auth_mock.get.return_value.status_code = 200
    auth_mock.get.return_value.json.return_value = {'results': []}

    netorca = Netorca(auth=auth_mock)
    result = netorca.get_deployed_items()

    assert result == []
    auth_mock.get.assert_called_once_with(url=ANY, authentication_required=True, filters=None)


def test_get_deployed_items_with_filters(auth_mock):
    auth_mock.get.return_value.status_code = 200
    auth_mock.get.return_value.json.return_value = {'results': []}

    filters = {'some_filter': 'some_value'}

    netorca = Netorca(auth=auth_mock)
    result = netorca.get_deployed_items(filters=filters)

    assert result == []
    auth_mock.get.assert_called_once_with(url=ANY, authentication_required=True, filters=filters)


def test_get_deployed_items_error(auth_mock):
    auth_mock.get.return_value.status_code = 500
    auth_mock.get.return_value.json.return_value = {'error': 'Some error'}

    netorca = Netorca(auth=auth_mock)

    with pytest.raises(NetorcaException, match='Could not fetch Deployed Items due to:'):
        netorca.get_deployed_items()


def test_get_change_instances_no_filters(auth_mock):
    auth_mock.get.return_value.status_code = 200
    auth_mock.get.return_value.json.return_value = {'results': []}

    netorca = Netorca(auth=auth_mock)
    result = netorca.get_change_instances()

    assert result == []
    auth_mock.get.assert_called_once_with(url=ANY, authentication_required=True, filters=None)


def test_get_change_instances_with_filters(auth_mock):
    auth_mock.get.return_value.status_code = 200
    auth_mock.get.return_value.json.return_value = {'results': []}

    filters = {'some_filter': 'some_value'}

    netorca = Netorca(auth=auth_mock)
    result = netorca.get_change_instances(filters=filters)

    assert result == []
    auth_mock.get.assert_called_once_with(url=ANY, authentication_required=True, filters=filters)


def test_get_change_instances_error(auth_mock):
    auth_mock.get.return_value.status_code = 500
    auth_mock.get.return_value.json.return_value = {'error': 'Some error'}

    netorca = Netorca(auth=auth_mock)

    with pytest.raises(NetorcaException, match='Could not fetch Change Instances due to:'):
        netorca.get_change_instances()


def test_create_deployed_item_success(auth_mock):
    change_instance_uuid = '12345678-1234-1234-1234-123456789012'
    data = {'field': 'value'}

    auth_mock.patch.return_value.status_code = 200
    auth_mock.patch.return_value.json.return_value = {'created': True}

    netorca = Netorca(auth=auth_mock)
    result = netorca.create_deployed_item(change_instance_uuid, data)

    assert result == {'created': True}
    auth_mock.patch.assert_called_once_with(
        url=ANY, data=json.dumps({'deployed_item': data}), authentication_required=True
    )


def test_create_deployed_item_error(auth_mock):
    change_instance_uuid = '12345678-1234-1234-1234-123456789012'
    data = {'field': 'value'}

    auth_mock.patch.return_value.status_code = 500
    auth_mock.patch.return_value.json.return_value = {'error': 'Some error'}

    netorca = Netorca(auth=auth_mock)

    with pytest.raises(NetorcaException, match='Could not create deployed item. Log:'):
        netorca.create_deployed_item(change_instance_uuid, data)


def test_update_change_instance_success(auth_mock):
    change_instance_uuid = '12345678-1234-1234-1234-123456789012'
    data = {'field': 'value'}

    auth_mock.patch.return_value.status_code = 200
    auth_mock.patch.return_value.json.return_value = {'updated': True}

    netorca = Netorca(auth=auth_mock)
    result = netorca.update_change_instance(change_instance_uuid, data)

    assert result == {'updated': True}
    auth_mock.patch.assert_called_once_with(
        url=ANY, data=json.dumps(data), authentication_required=True
    )


def test_update_change_instance_error(auth_mock):
    change_instance_uuid = '12345678-1234-1234-1234-123456789012'
    data = {'field': 'value'}

    auth_mock.patch.return_value.status_code = 500
    auth_mock.patch.return_value.json.return_value = {'error': 'Some error'}

    netorca = Netorca(auth=auth_mock)

    with pytest.raises(NetorcaException, match='Could not update change Instance. Log:'):
        netorca.update_change_instance(change_instance_uuid, data)



import json

from netorca_sdk.auth import AbstractNetorcaAuth
from netorca_sdk.config import SERVICE_ITEMS_ENDPOINT, DEPLOYED_ITEMS_ENDPOINT, CHANGE_INSTANCES_ENDPOINT, \
    SERVICE_CONFIG_ENDPOINT, logger
from netorca_sdk.exceptions import NetorcaException


class Netorca:
    def __init__(self, auth: AbstractNetorcaAuth):
        self.auth = auth

    def get_service_items(self, filters=None):
        """
        Fetches service items from the Netorca API based on the provided filters.

        :param filters: Optional dictionary containing filters to apply to the API request.
                        Example format: {"service_name": "my_service"}
        :type filters: dict, optional

        :return: A list of service items matching the provided filters.
        :rtype: list

        :raises NetorcaException: If the API request fails or encounters an error.
        """
        logger.info(f"Fetching service items with filters: {filters}")

        SERVICE_ITEMS_URL = f"{self.auth.fqdn}{SERVICE_ITEMS_ENDPOINT}"
        response = self.auth.get(url=SERVICE_ITEMS_URL, authentication_required=True, filters=filters)
        if response.status_code == 200:
            return response.json()['results']

        logger.error(f"Could not fetch Service Items due to: {response.json()}")
        raise NetorcaException(f"Could not fetch Service Items due to: {response.json()}")

    def get_deployed_items(self, filters=None):
        """
        Fetches deployed items from the Netorca API based on the provided filters.

        :param filters: Optional dictionary containing filters to apply to the API request.
        :type filters: dict, optional

        :return: A list of deployed items matching the provided filters.
        :rtype: list

        :raises NetorcaException: If the API request fails or encounters an error.
        """
        logger.info(f"Fetching deployed items with filters: {filters}")

        DEPLOYED_ITEMS_URL = f"{self.auth.fqdn}{DEPLOYED_ITEMS_ENDPOINT}"
        response = self.auth.get(url=DEPLOYED_ITEMS_URL, authentication_required=True, filters=filters)
        if response.status_code == 200:
            return response.json()['results']

        logger.error(f"Could not fetch Deployed Items due to: {response.json()}")
        raise NetorcaException(f"Could not fetch Deployed Items due to: {response.json()}")

    def get_change_instances(self, filters=None):
        """
        Fetches change instances from the Netorca API based on the provided filters.

        :param filters: Optional dictionary containing filters to apply to the API request.
                        Example format: {"service_name": "my_service1,my_service2"}
        :type filters: dict, optional

        :return: A list of change instances matching the provided filters.
        :rtype: list

        :raises NetorcaException: If the API request fails or encounters an error.
        """
        logger.info(f"Fetching change instances with filters: {filters}")

        CHANGE_INSTANCES_URL = f"{self.auth.fqdn}{CHANGE_INSTANCES_ENDPOINT}"
        response = self.auth.get(url=CHANGE_INSTANCES_URL, authentication_required=True, filters=filters)
        if response.status_code == 200:
            return response.json()['results']

        logger.error(f"Could not fetch Change Instances due to: {response.json()}")
        raise NetorcaException(f"Could not fetch Change Instances due to: {response.json()}")

    def create_deployed_item(self, change_instance_uuid: str, data: dict):
        """
        Creates a Deployed Item for a specified Change Instance in the Netorca API.

        :param change_instance_uuid: UUID of the ChangeInstance to which the DeployedItem should be added.
        :type change_instance_uuid: str

        :param data: Dictionary containing the metadata of the DeployedItem to be created.
                     Example format: {"key": "value"}
        :type data: dict

        :return: A dictionary containing the updated ChangeInstance with the new DeployedItem.
        :rtype: dict

        :raises NetorcaException: If the API request fails or encounters an error.
        """
        logger.info(f"Creating deployed item for change_instance_uuid: {change_instance_uuid} with data: {data}")

        CHANGE_INSTANCES_URL = f"{self.auth.fqdn}{CHANGE_INSTANCES_ENDPOINT}{change_instance_uuid}/"
        data = {
            'deployed_item': data
        }
        response = self.auth.patch(url=CHANGE_INSTANCES_URL, data=json.dumps(data), authentication_required=True)
        if response.status_code == 200:
            return response.json()

        logger.error(f"Could not create deployed item. Log: {response.json()}")
        raise NetorcaException(f"Could not create deployed item. Log: {response.json()}")

    def update_change_instance(self, change_instance_uuid, data):
        """
        Update a Deployed Item for a specified Change Instance in the Netorca API.

        :param change_instance_uuid: UUID of the ChangeInstance to which the DeployedItem should be added.
        :type change_instance_uuid: str

        :param data: Dictionary containing the metadata of the DeployedItem to be created.
                     Example format: {"key": "value"}
        :type data: dict

        :return: A dictionary containing the updated ChangeInstance with the new DeployedItem.
        :rtype: dict

        :raises NetorcaException: If the API request fails or encounters an error.
        """
        logger.info(f"Updating change instance with UUID: {change_instance_uuid} and data: {data}")

        CHANGE_INSTANCES_URL = f"{self.auth.fqdn}{CHANGE_INSTANCES_ENDPOINT}{change_instance_uuid}/"
        response = self.auth.patch(url=CHANGE_INSTANCES_URL, data=json.dumps(data), authentication_required=True)
        if response.status_code == 200:
            return response.json()

        logger.error(f"Could not update Change Instance. Log: {response.json()}")
        raise NetorcaException(f"Could not update change Instance. Log: {response.json()}")

    def get_service_config(self, config_uuid=None, filters=None):
        """
        Fetches service configuration(s) from the Netorca API based on the provided config_uuid or filters.

        :param config_uuid: Optional UUID of the specific service configuration to fetch.
                            If provided, filters will be ignored.
        :type config_uuid: str, optional

        :param filters: Optional dictionary containing filters to apply to the API request.
                        Example format: {"config_name": "PR019_IIW_001_config"}
                        Ignored if config_uuid is provided.
        :type filters: dict, optional

        :return: A dictionary containing the service configuration(s) matching the provided config_uuid or filters.
                 If config_uuid is provided, a single configuration will be returned; otherwise, a list of configurations.
        :rtype: dict or list

        :raises NetorcaException: If the API request fails or encounters an error.
        """
        logger.info(f"Fetching service config with config_uuid: {config_uuid} and filters: {filters}")

        SERVICE_CONFIG_URL = f"{self.auth.fqdn}{SERVICE_CONFIG_ENDPOINT}"
        if config_uuid:
            SERVICE_CONFIG_URL = f"{self.auth.fqdn}{SERVICE_CONFIG_ENDPOINT}{config_uuid}"

        response = self.auth.get(url=SERVICE_CONFIG_URL, authentication_required=True, filters=filters)
        if response.status_code == 200:
            return response.json()
        logger.error(f"Could not get service config. Log: {response.json()}")
        raise NetorcaException(f"Could not get service config. Log: {response.json()}")



import json
import os
from collections import defaultdict
from string import Template
from typing import Tuple

import requests
from beautifultable import BeautifulTable

from netorca_sdk.auth import AbstractNetorcaAuth, NetorcaAuth
from netorca_sdk.config import SUBMIT_SERVICEOWNER_SUBMISSION_DOCS_ENDPOINT, SUBMIT_SERVICEOWNER_SUBMISSION_ENDPOINT, \
    VALIDATE_SERVICEOWNER_SUBMISSION_ENDPOINT
from netorca_sdk.exceptions import NetorcaException


class ServiceOwnerSubmission:
    def __init__(self, netorca_api_key: str):
        self.netorca_api_key = netorca_api_key

        self.config = None
        self.serviceowner_submission = None
        self.auth = None

    def load_from_repository(self, repository_path: str) -> None:
        """
            Check if valid and load submission and config from service owner's repository.
            Repository must contain `.netorca` directory and `config.json` file.
            Note: Two allowed extensions in `.netorca` directory are: `*.yaml` and `*.md`.

            Args:
                repository_path: str    path to service owner repository

            Returns: None
        """
        repository_exists = os.path.isdir(repository_path)
        if not repository_exists:
            raise NetorcaException(
                f"`{repository_path}` directory does not exist.")
        netorca_exists = os.path.isdir(f"{repository_path}/.netorca")
        if not netorca_exists:
            raise NetorcaException("`.netorca` directory does not exist.")

        dotnetorca_path = f"{repository_path}/.netorca"

        # check and load config
        with open(f"{repository_path}/.netorca/config.json", "r") as stream:
            try:
                config = json.load(stream)
                netorca_global = config.get("netorca_global", {})
                if not (netorca_global or netorca_global.get("base_url")):
                    raise NetorcaException(
                        "No `netorca_global.base_url` provided.")

                self.config = config
                self.auth = self.get_auth()
            except json.JSONDecodeError as exc:
                raise NetorcaException(
                    f"Error while parsing file: `config.json`. Exception: {exc.msg}")

        _tmp_serviceowner_submission = defaultdict()
        # check and load service owner submission
        for filename in os.listdir(dotnetorca_path):
            if filename == "config.json":
                continue

            f = os.path.join(dotnetorca_path, filename)
            # checking if it is a file and is `*.json` or `*.md` file
            if not os.path.isfile(f) and not (f.endswith('.json') or f.endswith('.md')):
                raise NetorcaException(
                    f"ServiceOwner submission file: `{f}` does not exist or must be `.json` or `.md` extension")

            with open(f, "r") as stream:
                try:
                    filename_without_ext = os.path.splitext(filename)[0]

                    if filename.endswith('.json'):
                        json_file = json.load(stream)
                        _tmp_serviceowner_submission.setdefault(filename_without_ext, {})
                        _tmp_serviceowner_submission[filename_without_ext]["service"] = json_file
                except json.JSONDecodeError as exc:
                    raise NetorcaException(
                        f"Error while parsing file: `{filename}`. Exception: {exc}")

        for filename in os.listdir(dotnetorca_path):
            if filename == "config.json" or not filename.endswith('.md'):
                continue

            f = os.path.join(dotnetorca_path, filename)
            filename_without_ext = os.path.splitext(filename)[0]

            if not _tmp_serviceowner_submission.get(filename_without_ext):
                raise NetorcaException(
                    f"'\nYou are trying to add README file: `{filename}` for non existing service.\n"
                    f"Readme file (.md) must have the same name as service definition file (.json).")

            _tmp_serviceowner_submission[filename_without_ext]["readme"] = f

        self.serviceowner_submission = _tmp_serviceowner_submission

    def get_auth(self) -> AbstractNetorcaAuth:
        if not self.config:
            raise NetorcaException(f"Cannot authenticate before loading repository config.")

        netorca_fqdn = self.config.get("netorca_global", {}).get("base_url")
        self.auth = NetorcaAuth(fqdn=netorca_fqdn, api_key=self.netorca_api_key)
        return self.auth

    def get_team(self) -> dict:
        teams = self.auth.get_teams_info()
        if teams:
            return teams[0]
        return {}

    def validate(self, pretty_print=False) -> Tuple[bool, str]:
        """
            Validate service owner submission.
            NOTE: Data must be first imported with `load_from_repository` method

            Returns:
                bool    ->  validation successful
        """
        VALIDATE_SERVICEOWNER_PATH = f"{self.auth.fqdn}{VALIDATE_SERVICEOWNER_SUBMISSION_ENDPOINT}"
        invalid_services = []

        if not (self.config and self.serviceowner_submission and self.auth):
            raise NetorcaException(f"Use `load_from_repository(repository_path)` method to load configuration.")

        for service, service_submission in self.serviceowner_submission.items():
            response = self.auth.post(url=VALIDATE_SERVICEOWNER_PATH,
                                      data=json.dumps(service_submission['service']),
                                      authentication_required=True)
            response = response.json()
            if response.get("is_valid"):
                print(f"VALIDATION SUCCESSFUL for service: `{service}`.")
            else:
                invalid_services.append(service)
                errors = response.get("errors")
                if pretty_print and errors:
                    ServiceOwnerSubmission.pretty_print_errors(service, errors)

        if invalid_services:
            print("INVALID SERVICES: " + ", ".join(invalid_services))
            return False, "Invalid services: " + ", ".join(invalid_services)
        return True, "Services validated successfully."

    def submit(self) -> Tuple[bool, str]:
        """
            Validate and submit consumer submission.
            NOTE: Data must be first imported with `load_from_repository` method

            Returns:
                bool, str    ->  is submission successful, submission messages
        """
        SUBMIT_SERVICEOWNER_REQUEST_PATH = f"{self.auth.fqdn}{SUBMIT_SERVICEOWNER_SUBMISSION_ENDPOINT}"
        submitted_services = []
        is_valid = self.validate(pretty_print=True)

        if not is_valid[0]:
            return False, "Some of your submissions are invalid and were not submitted."

        print()
        for service, service_submission in self.serviceowner_submission.items():
            response = self.auth.post(url=SUBMIT_SERVICEOWNER_REQUEST_PATH,
                                      data=json.dumps(service_submission['service']),
                                      authentication_required=True)

            if response.status_code == 201:
                print(f"SUBMITTED service to NetOrca. Service: `{service}` is now available for consumers.")
                submitted_services.append(service)
                if service_submission.get("readme"):
                    service_uuid = response.json()['uuid']
                    docs_path = Template(SUBMIT_SERVICEOWNER_SUBMISSION_DOCS_ENDPOINT).substitute(uuid=service_uuid)
                    SUBMIT_SERVICEOWNER_DOCS_PATH = f"{self.auth.fqdn}{docs_path}"
                    files = {'md_file': ('VM.md', open(service_submission.get("readme"), 'rb'))}
                    auth = self.auth.headers['Authorization']
                    response = requests.post(SUBMIT_SERVICEOWNER_DOCS_PATH,
                                             files=files,
                                             headers={'Authorization': auth},
                                             verify=False)
                    if response.status_code == 200:
                        print(f"SUBMITTED README file for service: `{service}`.")
                    else:
                        print(f"FAILED to submit README file for service: `{service}`. Reason: `{response.json()}.`")
                else:
                    print(f"NOTE: Service `{service}` does not have a README file. Skipped.")
            else:
                print(f"FAILED to submit service: `{service}`. Reason: {response.json()}")
                print("\nMoving to next service...")
            print()

        if submitted_services:
            return True, "Submitted services: " + ", ".join(submitted_services)
        return False, "No services were submitted."

    @staticmethod
    def pretty_print_errors(service_name: str, errors: dict) -> None:
        """
            Pretty print errors
            #TODO: this should be refactored to cleaner code (probably recursive)
        """
        table = BeautifulTable(maxwidth=100)
        table.set_style(BeautifulTable.STYLE_SEPARATED)
        table.columns.header = ["Schema", "Property", "Reason"]
        for item1, value1 in errors.items():
            if isinstance(value1, str) or isinstance(value1, list):
                table.rows.append([service_name, "general", value1])
            elif isinstance(value1, dict):
                for item2, value2 in value1.items():
                    if isinstance(value2, str) or isinstance(value2, list):
                        table.rows.append([service_name, item2, value2])
            if table.rows:
                print("-" * 100)
                print(f"Schema: `{service_name}` validation errors")
                print("-" * 100)
                print(table)
                print()



import json
import os
from typing import Tuple

import yaml
from beautifultable import BeautifulTable

from netorca_sdk.auth import AbstractNetorcaAuth, NetorcaAuth
from netorca_sdk.config import VALIDATE_CONSUMER_SUBMISSION_ENDPOINT, SUBMIT_CONSUMER_SUBMISSION_ENDPOINT
from netorca_sdk.exceptions import NetorcaException


class ConsumerSubmission:
    def __init__(self, netorca_api_key: str):
        self.netorca_api_key = netorca_api_key

        self.config = None
        self.consumer_submission = None
        self.auth = None

    def load_from_repository(self, repository_path: str) -> None:
        """
            Check if valid and load request and config from consumer's repository.
            Repository must contain `.netorca` directory and `config.yaml` file.
            Note: Only one allowed extensions in `.netorca` directory is `*.yaml`

            Args:
                repository_path: str    path to consumer repository

            Returns: None
        """
        repository_exists = os.path.isdir(repository_path)
        if not repository_exists:
            raise NetorcaException(
                f"`{repository_path}` directory does not exist.")
        netorca_exists = os.path.isdir(f"{repository_path}/.netorca")
        if not netorca_exists:
            raise NetorcaException("`.netorca` directory does not exist.")

        dotnetorca_path = f"{repository_path}/.netorca"

        # check and load config
        with open(f"{repository_path}/.netorca/config.yaml", "r") as stream:
            try:
                config = yaml.safe_load(stream)
                netorca_global = config.get("netorca_global", {})
                if not (netorca_global or netorca_global.get("base_url")):
                    raise NetorcaException(
                        "No `netorca_global.base_url` provided.")

                self.config = config
                self.auth = self.get_auth()
            except yaml.YAMLError as exc:
                raise NetorcaException(
                    f"Error while parsing file: `config.yml`. Exception: {exc.problem}")

        _tmp_consumer_submission = {}
        # check and load consumer request
        for filename in os.listdir(dotnetorca_path):
            if filename == "config.yaml":
                continue

            f = os.path.join(dotnetorca_path, filename)
            # checking if it is a file and is `*.yaml`
            if not (os.path.isfile(f) and f.endswith('.yaml')):
                raise NetorcaException(
                    f"Consumer request files must have `.yaml` extension")

            with open(f, "r") as stream:
                try:
                    app = yaml.safe_load(stream)
                except yaml.YAMLError as exc:
                    raise NetorcaException(
                        f"Error while parsing file: `{filename}`. Exception: {exc.problem}")

                for key in app.keys():
                    if key in _tmp_consumer_submission:
                        raise NetorcaException(
                            f"Application with name `{key}` already exists in different `.yaml` declaration.")

                _tmp_consumer_submission.update(app)
        self.consumer_submission = _tmp_consumer_submission

    def get_auth(self) -> AbstractNetorcaAuth:
        if not self.config:
            raise NetorcaException(f"Cannot authenticate before loading repository config.")

        netorca_fqdn = self.config.get("netorca_global", {}).get("base_url")
        self.auth = NetorcaAuth(fqdn=netorca_fqdn, api_key=self.netorca_api_key)
        return self.auth

    def get_team(self) -> dict:
        teams = self.auth.get_teams_info()
        if teams:
            return teams[0]
        return {}

    def prepare_request(self) -> dict:
        team = self.get_team()
        metadata = self.config.get("netorca_global", {}).get("metadata", {})

        if not (team and self.config and self.consumer_submission and self.auth):
            raise NetorcaException(
                "Team, config and consumer request should be fetched at this stage.")

        full_request = {
            team['name']: {
                "metadata": metadata,
                **self.consumer_submission
            }
        }
        return full_request

    def validate(self, pretty_print=False) -> Tuple[bool, dict]:
        """
            Validate consume request.
            NOTE: Data must be first imported with `load_from_repository` method

            Returns:
                Tuple[bool, str]    ->  is_valid, validation_errors
        """
        if not (self.config and self.consumer_submission and self.auth):
            raise NetorcaException(f"Use `load_from_repository(repository_path)` method to load configuration.")
        VALIDATE_REQUEST_PATH = f"{self.auth.fqdn}{VALIDATE_CONSUMER_SUBMISSION_ENDPOINT}"
        full_request = self.prepare_request()

        response = self.auth.post(url=VALIDATE_REQUEST_PATH, data=json.dumps(full_request),
                                  authentication_required=True)

        response = response.json()
        if response.get("is_valid"):
            return True, {}
        errors = response.get("errors")

        if pretty_print:
            ConsumerSubmission.pretty_print_errors(errors)
        return False, errors

    def submit(self) -> Tuple[bool, str]:
        """
            Validate and submit consumer request.
            NOTE: Data must be first imported with `load_from_repository` method

            Returns:
                bool, str    ->  submission successful, submission messages
        """
        is_valid = self.validate(pretty_print=True)
        if not is_valid[0]:
            return False, "Consumer request is invalid and cannot be submitted."

        SUBMIT_REQUEST_PATH = f"{self.auth.fqdn}{SUBMIT_CONSUMER_SUBMISSION_ENDPOINT}"
        full_request = self.prepare_request()
        response = self.auth.post(url=SUBMIT_REQUEST_PATH, data=json.dumps(full_request),
                                  authentication_required=True)
        if response.status_code == 201:
            return True, "Submitted successfuly."
        return False, response.text

    @staticmethod
    def pretty_print_errors(errors: dict) -> None:
        """
            Pretty print errors
            #TODO: this should be refactored to cleaner code (probably recursive)
        """

        table = BeautifulTable(maxwidth=100)
        table.set_style(BeautifulTable.STYLE_SEPARATED)
        table.columns.header = ["Team", "Field", "Reason"]
        for item1, value1 in errors.items():
            if isinstance(value1, str) or isinstance(value1, list):
                table.rows.append([item1, "", value1])
            elif isinstance(value1, dict):
                for item2, value2 in value1.items():
                    if isinstance(value2, str) or isinstance(value2, list):
                        table.rows.append([item1, item2, value2])

                        if table.rows:
                            print("-" * 100)
                            print(f"Team: {item1} validation errors")
                            print("-" * 100)
                            print(table)
                            print()
                        break

        for item1, value1 in errors.items():
            if isinstance(value1, dict):
                for item2, value2 in value1.items():
                    table = BeautifulTable(maxwidth=100)
                    table.set_style(BeautifulTable.STYLE_SEPARATED)
                    table.columns.header = ["Application", "Service", "ServiceItem", "Field", "Reason"]

                    if isinstance(value2, dict):
                        for item3, value3 in value2.items():
                            if isinstance(value3, str):
                                table.rows.append([item2, "", "", item3, value3])
                            elif isinstance(value3, list):
                                for err in value3:
                                    table.rows.append([item2, "", "", item3, err])
                            elif isinstance(value3, dict):
                                for item4, value4 in value3.items():
                                    if isinstance(value4, str) or isinstance(value4, list):
                                        table.rows.append([item2, item3, "", item4, value4])
                                    elif isinstance(value4, dict):
                                        for item5, value5 in value4.items():
                                            if isinstance(value5, str) or isinstance(value5, list):
                                                table.rows.append([item2, item3, item4, item5, value5])

                        if table.rows:
                            print("-" * 100)
                            print(f"Application: `{item2}` validation errors")
                            print("-" * 100)
                            print(table)
                            print()



from distutils.core import setup

setup(
    name='netorca_sdk',
    packages=['netorca_sdk'],
    version='0.0.13',
    license='MIT',
    description='A package for interacting with the NetOrca API',
    author='Scott Rowlandson',
    author_email='scott@netautomate.org',
    url='https://gitlab.com/netorca_public/netorca_sdk/',
    keywords=['netorca', 'orchestration', 'netautomate'],
    install_requires=[
        'beautifultable',
        'pyyaml',
        'requests',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
)



from abc import abstractmethod, ABC
from time import sleep

import urllib3
import requests

from netorca_sdk.config import RETRY_TIMES, AUTH_ENDPOINT, TEAM_ENDPOINT
from netorca_sdk.exceptions import NetorcaException

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class AbstractNetorcaAuth(ABC):
    @abstractmethod
    def get(self, *args, **kwargs):
        ...

    @abstractmethod
    def post(self, *args, **kwargs):
        ...

    @abstractmethod
    def put(self, *args, **kwargs):
        ...

    @abstractmethod
    def patch(self, *args, **kwargs):
        ...

    @abstractmethod
    def delete(self, *args, **kwargs):
        ...


class NetorcaAuth(AbstractNetorcaAuth):
    def __init__(self, fqdn, username=None, password=None, api_key=None):
        self.username = username
        self.password = password
        self.api_key = api_key
        self.fqdn = fqdn
        self.headers = {
            'content-type': 'application/json'
        }

        if self.username and self.password:
            self.token = self.get_auth_token()
            self.headers['Authorization'] = f'Token {self.token}'
        elif self.api_key:
            self.headers['Authorization'] = f'Api-Key {self.api_key}'
        else:
            raise NetorcaException(
                f"Failed to authenticate. You must provide either (username and password) or API KEY")

    def get(self, *args, **kwargs):
        url = kwargs.get("url")
        filters = kwargs.get("filters")
        authentication_required = kwargs.get("authentication_required", False)

        if not url:
            raise NetorcaException(f"URL not provided!")

        for retry in range(RETRY_TIMES):
            if authentication_required:
                response = requests.get(url=url, headers=self.headers, verify=False, params=filters)
            else:
                response = requests.get(url=url, verify=False, params=filters)
            if 200 <= response.status_code < 500:
                return response
            sleep(retry)
        raise NetorcaException(
            f"Failed to send GET request. GET details: {args, kwargs}. Response details: {response.json()}")

    def post(self, *args, **kwargs):
        url = kwargs.get("url")
        data = kwargs.get("data")
        authentication_required = kwargs.get("authentication_required", False)

        if not url or not data:
            raise NetorcaException(f"URL or data not provided!")

        if authentication_required:
            response = requests.post(url=url, data=data, headers=self.headers, verify=False)
        else:
            response = requests.post(url=url, data=data, verify=False)

        if 200 <= response.status_code < 500:
            return response
        raise NetorcaException(
            f"Failed to send POST request. POST details: {args, kwargs}. Response details: {response.json()}")

    def put(self, *args, **kwargs):
        url = kwargs.get("url")
        data = kwargs.get("data")
        authentication_required = kwargs.get("authentication_required", False)

        if not url or not data:
            raise NetorcaException(f"URL or data not provided!")

        if authentication_required:
            response = requests.put(url=url, data=data, headers=self.headers, verify=False)
        else:
            response = requests.put(url=url, data=data, verify=False)
        if 200 <= response.status_code < 500:
            return response
        raise NetorcaException(
            f"Failed to send PUT request. PUT details: {args, kwargs}. Response details: {response.json()}")

    def patch(self, *args, **kwargs):
        url = kwargs.get("url")
        data = kwargs.get("data")
        authentication_required = kwargs.get("authentication_required", False)

        if not url or not data:
            raise NetorcaException(f"URL or data not provided!")

        if authentication_required:
            response = requests.patch(url=url, data=data, headers=self.headers, verify=False)
        else:
            response = requests.patch(url=url, data=data, verify=False)
        if 200 <= response.status_code < 500:
            return response
        raise NetorcaException(
            f"Failed to send PATCH request. PATCH details: {args, kwargs}. Response details: {response.json()}")

    def delete(self, *args, **kwargs):
        NetorcaException("Not implemented.")

    def get_auth_token(self) -> str:
        AUTH_URL = f"{self.fqdn}{AUTH_ENDPOINT}"

        data = {
            "username": self.username,
            "password": self.password
        }
        response = self.post(url=AUTH_URL, data=data, verify=False)
        if response.status_code == 200:
            return response.json()['token']
        raise NetorcaException(f"Authentication failed due to: {response.json()}")

    def refresh_auth_token(self):
        return self.get_auth_token()

    def get_teams_info(self) -> list:
        """ Get team info for given user"""
        TEAM_URL = f"{self.fqdn}{TEAM_ENDPOINT}"
        response = self.get(url=TEAM_URL, authentication_required=True)
        if response.status_code == 200:
            return response.json()['results']
        raise NetorcaException(f"Error fetching team info.")

    def __str__(self):
        return f"Username: {self.username}, netorca instance: {self.fqdn}."

    def __repr__(self):
        return f"Username: {self.username}, netorca instance: {self.fqdn}."
